# Generated by Django 2.1.2 on 2018-11-02 14:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wechat_user', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='status',
            field=models.PositiveSmallIntegerField(default=0),
        ),
    ]
