# Generated by Django 2.1.2 on 2018-12-20 14:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wechat_user', '0003_userwxacode'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='userwxacode',
            name='data',
        ),
        migrations.AddField(
            model_name='userwxacode',
            name='image',
            field=models.FileField(default='', upload_to='wxacodes', verbose_name='WXA Code'),
            preserve_default=False,
        ),
    ]
