# Generated by Django 2.1.2 on 2018-12-20 13:17

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wechat_user', '0002_user_status'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserWxaCode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('data', models.BinaryField(max_length=262144, verbose_name='Data')),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('owner', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='wxa_code', related_query_name='wxa_code', to='wechat_user.User', verbose_name='User')),
            ],
        ),
    ]
